@@ -1,5 +1,5 @@
 #    Back In Time
-#    Copyright (C) 2008-2009 Oprea Dan
+#    Copyright (C) 2008-2009 Oprea Dan, Bart de Koning, Richard Bailey
 #
 #    This program is free software; you can redistribute it and/or modify
 #    it under the terms of the GNU General Public License as published by
@@ -47,6 +47,7 @@ from PyKDE4.kio import *
 import kde4tools
 import settingsdialog
 import snapshotsdialog
+import logviewdialog
 
 
 _=gettext.gettext
@@ -67,12 +68,20 @@ class MainWindow( KMainWindow ):
 		self.main_toolbar = self.toolBar()
 		self.main_toolbar.setFloatable( False )
 
+		#profiles
+		#self.main_toolbar.addWidget( QLabel( QString.fromUtf8( _('Profile:') ), self ) )
+		self.first_update_all = True
+		self.disable_profile_changed = False
+		self.combo_profiles = KComboBox( self )
+		self.combo_profiles_action = self.main_toolbar.addWidget( self.combo_profiles )
+		#self.combo_profiles.setMinimumWidth( 250 )
+
 		self.btn_take_snapshot = self.main_toolbar.addAction( KIcon( 'document-save' ), '' )
 		self.btn_take_snapshot.setToolTip( QString.fromUtf8( _('Take snapshot') ) )
 		QObject.connect( self.btn_take_snapshot, SIGNAL('triggered()'), self.on_btn_take_snapshot_clicked )
 
 		self.btn_update_snapshots = self.main_toolbar.addAction( KIcon( 'view-refresh' ), '' )
-		self.btn_update_snapshots.setToolTip( QString.fromUtf8( _('Update snapshots') ) )
+		self.btn_update_snapshots.setToolTip( QString.fromUtf8( _('Refresh snapshots list') ) )
 		QObject.connect( self.btn_update_snapshots, SIGNAL('triggered()'), self.on_btn_update_snapshots_clicked )
 
 		self.btn_name_snapshot = self.main_toolbar.addAction( KIcon( 'edit-rename' ), '' )
@@ -83,6 +92,14 @@ class MainWindow( KMainWindow ):
 		self.btn_remove_snapshot.setToolTip( QString.fromUtf8( _('Remove Snapshot') ) )
 		QObject.connect( self.btn_remove_snapshot, SIGNAL('triggered()'), self.on_btn_remove_snapshot_clicked )
 	
+		self.btn_snapshot_log_view = self.main_toolbar.addAction( KIcon( 'text-plain' ), '' )
+		self.btn_snapshot_log_view.setToolTip( QString.fromUtf8( _('View Snapshot Log') ) )
+		QObject.connect( self.btn_snapshot_log_view, SIGNAL('triggered()'), self.on_btn_snapshot_log_view_clicked )
+	
+		self.btn_log_view = self.main_toolbar.addAction( KIcon( 'document-new' ), '' )
+		self.btn_log_view.setToolTip( QString.fromUtf8( _('View Last Log') ) )
+		QObject.connect( self.btn_log_view, SIGNAL('triggered()'), self.on_btn_log_view_clicked )
+	
 		self.main_toolbar.addSeparator()
 
 		self.btn_settings = self.main_toolbar.addAction( KIcon( 'configure' ), '' )
@@ -112,7 +129,7 @@ class MainWindow( KMainWindow ):
 		self.btn_link = self.main_toolbar.addAction( KIcon( 'go-home' ), '' )
 		self.btn_link.setToolTip( QString.fromUtf8( 'backintime.le-web.org' ) )
 		QObject.connect( self.btn_link, SIGNAL('triggered()'), self.open_url )
-
+		
 		#main splitter
 		self.main_splitter = QSplitter( self )
 		self.main_splitter.setOrientation( Qt.Horizontal )
@@ -160,7 +177,7 @@ class MainWindow( KMainWindow ):
 		QObject.connect( self.btn_restore, SIGNAL('triggered()'), self.on_btn_restore_clicked )
 
 		self.btn_copy = self.files_view_toolbar.addAction( KIcon( 'edit-copy' ), '' )
-		self.btn_copy.setToolTip( QString.fromUtf8( _('Copy') ) )
+		self.btn_copy.setToolTip( QString.fromUtf8( _('Copy pathname to clipboard') ) )
 		QObject.connect( self.btn_copy, SIGNAL('triggered()'), self.on_btn_copy_to_clipboard_clicked )
 
 		self.btn_snapshots = self.files_view_toolbar.addAction( KIcon( 'view-list-details' ), '' )
@@ -187,7 +204,7 @@ class MainWindow( KMainWindow ):
 		self.second_splitter.addWidget( widget )
 
 		#folder don't exist label
-		self.lbl_folder_dont_exists = QLabel( QString.fromUtf8( _('This folder don\'t exist\nin current snapshot !') ), self )
+		self.lbl_folder_dont_exists = QLabel( QString.fromUtf8( _('This folder doesn\'t exist\nin the current snapshot !') ), self )
 		kde4tools.set_font_bold( self.lbl_folder_dont_exists )
 		self.lbl_folder_dont_exists.setFrameShadow( QFrame.Sunken )
 		self.lbl_folder_dont_exists.setFrameShape( QFrame.Panel )
@@ -281,7 +298,7 @@ class MainWindow( KMainWindow ):
 		self.text_validator = QRegExpValidator( QRegExp( '.*' ), self )
 
 		#
-		kde4tools.set_font_bold( self.right_widget )
+		#kde4tools.set_font_bold( self.right_widget )
 
 		#force settingdialog if it is not configured
 		if not cfg.is_configured():
@@ -289,17 +306,23 @@ class MainWindow( KMainWindow ):
 
 		if not cfg.is_configured():
 			return
+	
+		if self.snapshots.has_old_snapshots():
+			settingsdialog.SettingsDialog( self ).update_snapshots_location()
 
-		if not cfg.can_backup():
+		profile_id = cfg.get_current_profile()
+		if not cfg.can_backup( profile_id ):
 			KMessageBox.error( self, QString.fromUtf8( _('Can\'t find snapshots folder.\nIf it is on a removable drive please plug it and then press OK') ) )
 
 		QObject.connect( self.list_files_view_model.dirLister(), SIGNAL('completed()'), self.on_dir_lister_completed )
-		QObject.connect( self.list_files_view_model.dirLister(), SIGNAL('canceled()'), self.on_dir_lister_completed )
+		QObject.connect( self.list_files_view_model.dirLister(), SIGNAL('cancelled()'), self.on_dir_lister_completed )
 
 		#populate lists
-		self.update_time_line()
-		self.update_places()
-		self.update_files_view( 0 )
+		self.update_profiles()
+		QObject.connect( self.combo_profiles, SIGNAL('currentIndexChanged(int)'), self.on_profile_changed )
+		#self.update_time_line()
+		#self.update_places()
+		#self.update_files_view( 0 )
 
 		self.list_files_view.setFocus()
 
@@ -352,6 +375,46 @@ class MainWindow( KMainWindow ):
 
 		event.accept()
 
+	def update_profiles( self ):
+		if self.disable_profile_changed:
+			return
+
+		self.disable_profile_changed = True
+
+		self.combo_profiles.clear()
+
+		index = 0
+		profiles = self.config.get_profiles_sorted_by_name()
+
+		for profile_id in profiles:
+			if profile_id == self.config.get_current_profile():
+				index = self.combo_profiles.count()
+			self.combo_profiles.addItem( QString.fromUtf8( self.config.get_profile_name( profile_id ) ), QVariant( QString.fromUtf8( profile_id ) ) )
+
+		self.combo_profiles.setCurrentIndex( index )
+		self.combo_profiles_action.setVisible( len( profiles ) > 1 )
+
+		self.update_profile()
+
+		self.disable_profile_changed = False
+
+	def update_profile( self ):
+		self.update_time_line()
+		self.update_places()
+		self.update_files_view( 0 )
+
+	def on_profile_changed( self, index ):
+		if self.disable_profile_changed:
+			return
+
+		profile_id = str( self.combo_profiles.itemData( index ).toString().toUtf8() )
+		if len( profile_id ) <= 0:
+			return
+		
+		if profile_id != self.config.get_current_profile():
+			self.config.set_current_profile( profile_id )
+			self.update_profile()
+
 	def get_default_startup_folder_and_file( self ):
 		last_path = self.config.get_str_value( 'gnome.last_path', '' )
 		if len(last_path) > 0 and os.path.isdir(last_path):
@@ -429,35 +492,49 @@ class MainWindow( KMainWindow ):
 
 		fake_busy = busy or self.force_wait_lock_counter > 0
 
+		take_snapshot_message = self.snapshots.get_take_snapshot_message()
 		if fake_busy:
-			take_snapshot_message = None
-
-			if busy:
-				take_snapshot_message = self.snapshots.get_take_snapshot_message()
-
 			if take_snapshot_message is None:
 				take_snapshot_message = ( 0, '...' )
+		elif take_snapshot_message is None:
+			take_snapshot_message = self.last_take_snapshot_message
+			if take_snapshot_message is None:
+				take_snapshot_message = ( 0, _('Done') )
 
-			if take_snapshot_message != self.last_take_snapshot_message:
-				self.last_take_snapshot_message = take_snapshot_message
-				self.statusBar().showMessage( QString.fromUtf8( _('Working:') + ' ' + self.last_take_snapshot_message[1].replace( '\n', ' ' ) ) )
+		force_update = False
 
+		if fake_busy:
 			if self.btn_take_snapshot.isEnabled():
 				self.btn_take_snapshot.setEnabled( False )
 		elif not self.btn_take_snapshot.isEnabled():
+			force_update = True
+
 			self.btn_take_snapshot.setEnabled( True )
 			
-			snapshots_list = self.snapshots.get_snapshots_list()
+			snapshots_list = self.snapshots.get_snapshots_and_other_list()
 
 			if snapshots_list != self.snapshots_list:
 				self.snapshots_list = snapshots_list
 				self.update_time_line( False )
-			 	self.statusBar().showMessage( QString.fromUtf8( _('Done') ) )
+				take_snapshot_message = ( 0, _('Done') )
 			else:
-				self.statusBar().showMessage( QString.fromUtf8( _('Done, no backup needed') ) )
+				if take_snapshot_message[0] == 0:
+					take_snapshot_message = ( 0, _('Done, no backup needed') )
 
-		if not fake_busy:
-			self.last_take_snapshot_message = None
+		if take_snapshot_message != self.last_take_snapshot_message or force_update:
+			self.last_take_snapshot_message = take_snapshot_message
+
+			if fake_busy:
+				message = _('Working:') + ' ' + self.last_take_snapshot_message[1].replace( '\n', ' ' )
+			elif take_snapshot_message[0] == 0:
+				message = self.last_take_snapshot_message[1].replace( '\n', ' ' )
+			else:
+				message = _('Error:') + ' ' + self.last_take_snapshot_message[1].replace( '\n', ' ' )
+
+			self.statusBar().showMessage( QString.fromUtf8( message ) )
+
+		#if not fake_busy:
+		#	self.last_take_snapshot_message = None
 
 	def on_list_places_current_item_changed( self, item, previous ):
 		if item is None:
@@ -503,11 +580,18 @@ class MainWindow( KMainWindow ):
 		self.add_place( QString.fromUtf8( _('Home') ), os.path.expanduser( '~' ), 'user-home' )
 
 		#add backup folders
-		include_folders = self.config.get_include_folders()
+		include_folders = self.config.get_include()
 		if len( include_folders ) > 0:
-			self.add_place( QString.fromUtf8( _('Backup folders') ), '', '' )
-			for folder in include_folders:
-				self.add_place( QString.fromUtf8(folder[0]), folder[0], 'document-save' )
+			folders = []
+			for item in include_folders:
+				if item[1] == 0:
+					folders.append( item[0] )
+
+			if len( folders ) > 0:
+				self.add_place( QString.fromUtf8( _('Backup folders') ), '', '' )
+				for folder in folders:
+					#self.add_place( QString.fromUtf8(folder[0]), folder[0], 'document-save' )
+					self.add_place( QString.fromUtf8(folder), folder, 'document-save' )
 
 	def update_snapshot_actions( self ):
 		enabled = False
@@ -520,6 +604,7 @@ class MainWindow( KMainWindow ):
 		#update remove/name snapshot buttons
 		self.btn_name_snapshot.setEnabled( enabled )
 		self.btn_remove_snapshot.setEnabled( enabled )
+		self.btn_snapshot_log_view.setEnabled( enabled )
 
 	def on_list_time_line_current_item_changed( self, item, previous ):
 		self.update_snapshot_actions()
@@ -567,7 +652,7 @@ class MainWindow( KMainWindow ):
 		self.add_time_line( QString.fromUtf8( self.snapshots.get_snapshot_display_name( '/' ) ), '/' )
 
 		if get_snapshots_list:
-			self.snapshots_list = self.snapshots.get_snapshots_list() 
+			self.snapshots_list = self.snapshots.get_snapshots_and_other_list() 
 
 		groups = []
 		now = datetime.date.today()
@@ -627,7 +712,7 @@ class MainWindow( KMainWindow ):
 				self.update_files_view( 2 )
 
 	def on_btn_take_snapshot_clicked( self ):
-		backintime.take_snapshot_now_async()
+		backintime.take_snapshot_now_async( self.config )
 		self.update_take_snapshot( True )
 
 	def on_btn_update_snapshots_clicked( self ):
@@ -656,6 +741,20 @@ class MainWindow( KMainWindow ):
 		self.snapshots.set_snapshot_name( snapshot_id, new_name )
 		self.time_line_update_snapshot_name( item )
 
+	def on_btn_log_view_clicked ( self ):
+		logviewdialog.LogViewDialog( self ).exec_()
+
+	def on_btn_snapshot_log_view_clicked ( self ):
+		item = self.list_time_line.currentItem()
+		if item is None:
+			return
+
+		snapshot_id = self.time_line_get_snapshot_id( item )
+		if len( snapshot_id ) <= 1:
+			return
+
+		logviewdialog.LogViewDialog( self, snapshot_id ).exec_()
+
 	def on_btn_remove_snapshot_clicked ( self ):
 		item = self.list_time_line.currentItem()
 		if item is None:
@@ -672,31 +771,19 @@ class MainWindow( KMainWindow ):
 		self.update_time_line()
 
 	def on_btn_settings_clicked( self ):
-		snapshots_path = self.config.get_snapshots_path()
-		include_folders = self.config.get_include_folders()
-
-		settingsdialog.SettingsDialog( self ).exec_()
-
-		update_files_view = ( include_folders != self.config.get_include_folders() )
-
-		if snapshots_path == self.config.get_snapshots_path() and not update_files_view:
-		   return
-
-		if update_files_view:
-			self.update_places()
-
-		if snapshots_path != self.config.get_snapshots_path():
-			self.update_time_line( True )
+		if QDialog.Accepted == settingsdialog.SettingsDialog( self ).exec_():
+			self.update_profiles()
 
 	def on_btn_about_clicked( self ):
 		dlg = KAboutApplicationDialog( self.kaboutdata, self )
 		dlg.exec_()
 
 	def on_btn_help_clicked( self ):
-		KRun.runCommand( "khelpcenter help:/backintime", self )
+		#KRun.runCommand( "khelpcenter help:/backintime", self )
+		self.open_url( 'http://backintime.le-web.org/documentation' );
 
 	def open_url( self, url = 'http://backintime.le-web.org' ):
-		self.run = KRun( KUrl( url ), self, True )
+		self.run = KRun( KUrl( QString.fromUtf8( url ) ), self, True )
 
 	def on_btn_show_hidden_files_toggled( self, checked ):
 		self.show_hidden_files = checked
@@ -706,7 +793,8 @@ class MainWindow( KMainWindow ):
 		if len( self.snapshot_id ) <= 1:
 			return
 
-		selected_file = str( self.list_files_view_sort_filter_proxy.data( self.list_files_view.currentIndex() ).toString().toUtf8() )
+		idx = self.list_files_view_sort_filter_proxy.index( self.list_files_view.currentIndex().row(), 0 )
+		selected_file = str( self.list_files_view_sort_filter_proxy.data( idx ).toString().toUtf8() )
 		if len( selected_file ) <= 0:
 			return
 
@@ -714,7 +802,8 @@ class MainWindow( KMainWindow ):
 		self.snapshots.restore( self.snapshot_id, rel_path )
 
 	def on_btn_copy_to_clipboard_clicked( self ):
-		selected_file = str( self.list_files_view_sort_filter_proxy.data( self.list_files_view.currentIndex() ).toString().toUtf8() )
+		idx = self.list_files_view_sort_filter_proxy.index( self.list_files_view.currentIndex().row(), 0 )
+		selected_file = str( self.list_files_view_sort_filter_proxy.data( idx ).toString().toUtf8() )
 		if len( selected_file ) <= 0:
 			return
 
@@ -722,14 +811,15 @@ class MainWindow( KMainWindow ):
 		kde4tools.clipboard_set_path( self.kapp, path )
 
 	def on_btn_snapshots_clicked( self ):
-		selected_file = str( self.list_files_view_sort_filter_proxy.data( self.list_files_view.currentIndex() ).toString().toUtf8() )
+		idx = self.list_files_view_sort_filter_proxy.index( self.list_files_view.currentIndex().row(), 0 )
+		selected_file = str( self.list_files_view_sort_filter_proxy.data( idx ).toString().toUtf8() )
 		if len( selected_file ) <= 0:
 			return
 
 		rel_path = os.path.join( self.path, selected_file )
 		icon = None
-		if self.list_files_view_sort_filter_proxy.data( self.list_files_view.currentIndex(), Qt.DecorationRole ).type() == QVariant.Icon:
-			icon = self.list_files_view_sort_filter_proxy.data( self.list_files_view.currentIndex(), Qt.DecorationRole )
+		if self.list_files_view_sort_filter_proxy.data( idx, Qt.DecorationRole ).type() == QVariant.Icon:
+			icon = self.list_files_view_sort_filter_proxy.data( idx, Qt.DecorationRole )
 
 		dlg = snapshotsdialog.SnapshotsDialog( self, self.snapshot_id, rel_path, icon )
 		if QDialog.Accepted == dlg.exec_():
@@ -761,18 +851,21 @@ class MainWindow( KMainWindow ):
 		if model_index is None:
 			return
 
-		rel_path = str( self.list_files_view_sort_filter_proxy.data( model_index ).toString().toUtf8() )
+		idx = self.list_files_view_sort_filter_proxy.index( model_index.row(), 0 )
+		rel_path = str( self.list_files_view_sort_filter_proxy.data( idx ).toString().toUtf8() )
 		if len( rel_path ) <= 0:
 			return
 
 		rel_path = os.path.join( self.path, rel_path )
 		full_path = self.snapshots.get_snapshot_path_to( self.snapshot_id, rel_path )
 
-		if os.path.isdir( full_path ):
-			self.path = rel_path
-			self.update_files_view( 0 )
-		else:
-			self.run = KRun( KUrl( full_path ), self, True )
+		if os.path.exists( full_path ):
+			if self.snapshots.can_open_path( self.snapshot_id, full_path ):
+				if os.path.isdir( full_path ):
+					self.path = rel_path
+					self.update_files_view( 0 )
+				else:
+					self.run = KRun( KUrl( QString.fromUtf8( full_path ) ), self, True )
 
 	def files_view_get_name( self, item ):
 		return str( item.text( 0 ).toUtf8() )
@@ -782,7 +875,7 @@ class MainWindow( KMainWindow ):
 
 	def add_files_view( self, name, size_str, date_str, size_int, type ):
 		full_path = self.snapshots.get_snapshot_path_to( self.snapshot_id, os.path.join( self.path, name ) )
-		icon = KIcon( KMimeType.iconNameForUrl( KUrl( full_path ) ) )
+		icon = KIcon( KMimeType.iconNameForUrl( KUrl( QString.fromUtf8( full_path ) ) ) )
 
 		item = QTreeWidgetItem( self.list_files_view )
 
@@ -824,7 +917,8 @@ class MainWindow( KMainWindow ):
 
 		#try to keep old selected file
 		if selected_file is None:
-			selected_file = str( self.list_files_view_sort_filter_proxy.data( self.list_files_view.currentIndex() ).toString().toUtf8() )
+			idx = self.list_files_view_sort_filter_proxy.index( self.list_files_view.currentIndex().row(), 0 )
+			selected_file = str( self.list_files_view_sort_filter_proxy.data( idx ).toString().toUtf8() )
 
 		self.selected_file = selected_file
 	
@@ -968,7 +1062,7 @@ class KDE4TakeSnapshotCallback( threading.Thread ): #used to display status icon
 
 
 def create_kapplication( cfg ):
-	kaboutdata = KAboutData( 'backintime', '', ki18n( cfg.APP_NAME ), cfg.VERSION, ki18n( '' ), KAboutData.License_GPL_V2, ki18n( cfg.COPYRIGHT ), ki18n( '' ), 'http://backintime.le-web.org', 'dan@le-web.org' )
+	kaboutdata = KAboutData( 'backintime', '', ki18n( cfg.APP_NAME ), cfg.VERSION, ki18n( '' ), KAboutData.License_GPL_V2, ki18n( cfg.COPYRIGHT ), ki18n( '' ), 'http://backintime.le-web.org', 'bit-team@lists.launchpad.net' )
 	kaboutdata.setProgramIconName( 'document-save' )
 
 	#extra_translations = _('about-translators')
